from .basecompiler import BaseCompiler

SUPPORTED_COMPILERS:dict[str, dict] = {
    "EMCC": BaseCompiler("EMCC", "emcc", ["wasm", "js", "html"]),
    "GCC": BaseCompiler("GCC", "gcc", ["exe", "dll", "dylib", "so"]),
    "CLANG": BaseCompiler("Clang", "clang", ["exe", "dll", "dylib", "so"])
}


import os, json
import platform
import subprocess

SEP = os.path.sep
def os_path(path:str=None) -> str:
    if path is None: return None
    return path.strip().replace("/", SEP).replace("\\", SEP)

def os_mkdir(path:str=None) -> None:
    if path is None: return None
    if not os.path.exists(path):
        root = path.split(SEP)[0]
        dirs = path.split(SEP)[1:]
        if not os.path.exists(root): os.mkdir(root)
        for d in dirs:
            root += f"{SEP}{d}"
            if not os.path.exists(root): os.mkdir(root)



from argparse import ArgumentParser

from rich.panel import Panel
from rich.console import Console

from r3make.build import r3make_build
from r3make.config import parse_config
from r3make.version import YEAR, MINOR, PATCH

def main():
    console = Console()
    parser = ArgumentParser(description="r3make - A lightweight C/C++ build tool")
    parser.add_argument('config', help="Path to the .r3make configuration file")
    r3make_build(console, parse_config(parser.parse_args().config))

if __name__ == "__main__":
    main()


import r3make.utils as utils
from r3make.compiler import SUPPORTED_COMPILERS, BaseCompiler

DEFAULT_SEARCH_PATHS: dict[str, list[str]] = {}

def define_search_paths() -> None:
    """
    Defines the default search paths for all supported compilers and platforms,
    including both archives and binaries.
    """
    global DEFAULT_SEARCH_PATHS

    # common os-specific default paths
    utils.os_default_paths = {
        "Windows": [
            utils.os_path("C:\\Windows\\System32"),
            utils.os_path("C:\\Windows\\System32\\wbem")
        ],
        "Linux": [
            utils.os_path("/lib"),
            utils.os_path("/usr/lib"),
            utils.os_path("/usr/local/lib")
        ],
        "Darwin": [
            utils.os_path("/lib"),
            utils.os_path("/usr/lib"),
            utils.os_path("/usr/local/lib"),
            utils.os_path("/opt/homebrew/lib")  # Homebrew on ARM macutils.OS
        ]
    }

    # os-specific defaults
    DEFAULT_SEARCH_PATHS["utils.OS_DEFAULTS"] = utils.os_default_paths.get(utils.platform.system(), [])

    for name, compiler in SUPPORTED_COMPILERS.items():
        DEFAULT_SEARCH_PATHS[name] = {
            "libraries": [],
            "programs": []
        }

        prefix = compiler.prefix
        search_command = f"{prefix} -print-search-dirs"

        try:
            result = utils.subprocess.run(
                search_command,
                text=True,
                check=True,
                capture_output=True,
                shell=True
            )
            output = result.stdout
        except (utils.subprocess.CalledProcessError, FileNotFoundError):
            # fallback to utils.os.popen if utils.subprocess fails (python and environment variables arent cool)
            try:
                with utils.os.popen(search_command) as proc:
                    output = proc.read()
            except FileNotFoundError:
                print(f"Compiler '{prefix}' not found in PATH.")
                continue

        for line in output.splitlines():
            if line.startswith("libraries: ="):
                paths = line.split("=", 1)[1].strip().split(utils.os.pathsep)
                DEFAULT_SEARCH_PATHS[name]["libraries"].extend(paths)
            elif line.startswith("programs: ="):
                paths = line.split("=", 1)[1].strip().split(utils.os.pathsep)
                DEFAULT_SEARCH_PATHS[name]["programs"].extend(paths)

def fetch_compiler_instance(compiler_name) -> BaseCompiler:
    # TODO: extend to fetch OS default compiler
    try:
        return SUPPORTED_COMPILERS[compiler_name]
    except (KeyError) as e:
        print(f"Unsupported compiler: {compiler_name}")
        return None

def fetch_files(extension:str, directory:str) -> list[str]:
    directory = utils.os_path(directory)
    source_files = []
    if not utils.os.path.exists(directory): return source_files
    for root, _, files in utils.os.walk(directory.strip()):
        for file in files:
            if file.endswith(extension):
                source_files.append(utils.os.path.join(root, file))
    return source_files

def fetch_library(lib:str, path: str) -> bool:
    """
    Recursively searches for library files (.dll, .a, .lib, .so) in the specified directory.
    Returns True if any library is found, otherwise False.
    """
    path = utils.os_path(path)
    if not utils.os.path.exists(path):
        return False

    define_search_paths()
    current_dirs = [utils.os.getcwd(), path]

    for search_path in DEFAULT_SEARCH_PATHS.values():
        current_dirs.extend(search_path)

    # Search recursively across all applicable directories
    for directory in set(current_dirs):
        for root, _, files in utils.os.walk(directory.strip()):
            for file in files:
                if file.endswith(('.dll', '.a', '.lib', '.so')) and file.split(".")[0] == lib:
                    return True



import r3make.utils as utils

CBUILD_DEFAULT_CONFIG:dict = {
    "r3make": {
        "pre-build": {},
        "post-build": {}
    },
    "c-instance": "GCC",
    "c-flags": [],
    "c-defines": [],
    "src-dirs": [],
    "inc-dirs": [],
    "lib-links": {},
    "out-dir": "bin",
    "out-type": "exe",
    "out-name": "program",
}

def parse_config(config_path:str) -> dict | None:
    if not utils.os.path.exists(config_path): return None

    with open(config_path, "rb") as f:
        config = utils.json.load(f)
        f.close()
    
    # parse the config and set default values
    pconfig = {key: config.get(key, CBUILD_DEFAULT_CONFIG[key]) for key in CBUILD_DEFAULT_CONFIG}
    pconfig["r3make"] = {k: config["r3make"].get(k, {}) for k in CBUILD_DEFAULT_CONFIG["r3make"]}
    
    # parse compiler instace configuration
    pconfig["c-flags"] = [ c_flag.strip() for c_flag in pconfig["c-flags"] ]
    pconfig["c-defines"] = [ c_define.strip() for c_define in pconfig["c-defines"] ]
    
    # parse source file configuration
    pconfig["src-dirs"] = [ utils.os_path(src_dir) for src_dir in pconfig["src-dirs"] ]
    
    # parse include file configuration
    pconfig["inc-dirs"] = [ utils.os_path(src_dir) for src_dir in pconfig["inc-dirs"] ]
    
    # parse output configuration
    pconfig["out-dir"] = utils.os_path(pconfig["out-dir"])
    
    for lib, lib_path in pconfig["lib-links"].items():
        pconfig["lib-links"][lib] = utils.os_path(lib_path) if isinstance(lib_path, str) else None

    return pconfig



from rich.panel import Panel
from rich.console import Console

import r3make.utils as utils

console = Console()
def _test_command(config:dict, param=None) -> None:
    console.print(Panel(f"[bold cyan]r3make command[/] {param}", expand=False))

def _nofiles_command(config:dict, param=None) -> None:
    console.print(Panel(f"[bold cyan]r3make command[/] Removing O-Files", expand=False))
    utils.os.system(f"rmdir {config["out-dir"]}{utils.SEP}ofiles /s /q")
    console.print(Panel(f"[bold cyan]r3make command[/] Removed O-Files", expand=False))

CBUILD_COMMANDS:dict = {
    "nofiles": _nofiles_command,
    "print-test": _test_command
}



from rich.console import Console
from rich.progress import Progress, BarColumn, SpinnerColumn

import r3make.utils as utils

class BaseCompiler:
    def __init__(
            self,
            name:str,
            prefix:str,
            out_types:list = []
        ):
        self.name:str = name
        self.prefix:str = prefix
        self.out_types:list = out_types

    def compile(self, c_flags: list, c_defines: list, src_files: list[str], inc_dirs: list[str], out_dir: str) -> list[str]:
        ofiles = []
        with Progress(SpinnerColumn(), "[progress.description]{task.description}", BarColumn(), transient=True) as progress:
            for source in progress.track(src_files, description=f"Compiling {self.name} source files..."):
                sourcefile = source.split(utils.SEP)[-1].replace(".c", ".o")
                ofile = f"{out_dir}{utils.SEP}ofiles{utils.SEP}{sourcefile}"
                
                flag_str = " ".join([f"{flag.strip()}" for flag in c_flags])
                define_str = " ".join([f"-D{define.strip()}" for define in c_defines])
                include_str = " ".join([f"-I{include.strip()}" for include in inc_dirs])

                command = f"{self.prefix} {flag_str} {define_str} {include_str} -c {source} -o {ofile}"
                try:
                    utils.subprocess.run(command, check=True)
                    ofiles.append(ofile)
                except FileNotFoundError:
                    utils.os.system(command)
                    ofiles.append(ofile)
                except (OSError, utils.subprocess.CalledProcessError) as e:
                    print(f"Error during Compile: {e}")
                    return None
        return ofiles

    def link(self, ofiles: list[str], lib_links: dict[str, str], out_name: str, out_type: str, out_dir: str) -> bool:
        ofile_str = " ".join(ofiles)
        lib_str = " ".join([f"-l{lib}" for lib in lib_links])
        lib_dir_str = " ".join([f"-L{lib_links[lib]}" for lib in lib_links if lib_links[lib]])

        out_ext = f".{out_type}" if out_type in self.out_types else ""

        platform_name = utils.platform.system().lower()
        if out_type in ["dll", "so", "dylib"]:
            if platform_name == "darwin":
                command = f"{self.prefix} -dynamiclib {ofile_str} {lib_str} {lib_dir_str} -o {out_dir}{utils.SEP}{out_name}{out_ext}"
            else:
                command = f"{self.prefix} -shared {ofile_str} {lib_str} {lib_dir_str} -o {out_dir}{utils.SEP}{out_name}{out_ext}"

        elif out_type == "exe":
            command = f"{self.prefix} {ofile_str} {lib_str} {lib_dir_str} -o {out_dir}{utils.SEP}{out_name}{out_ext}"

        try:
            utils.subprocess.run(command, check=True)
            return True
        except FileNotFoundError:
            utils.os.system(command)
            return True
        except (OSError, utils.subprocess.CalledProcessError) as e:
            print(f"Error during Linking: {e}")
            return False



import setuptools

version = {
    "year" :2025,
    "minor" :1,
    "patch" :1
}

setuptools.setup(
    name='r3make',
    version=f"{version["year"]}.{version["minor"]}.{version["patch"]}",
    description='A command-line build tool for C without the bloat of CMake.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Izaiyah Stokes',
    author_email='d34d0s.dev@gmail.com',
    url='https://github.com/r3shape/r3make',
    packages=setuptools.find_packages(),
    install_requires=[
        "rich"
    ],
    entry_points={
        'console_scripts': [
            'r3make = r3make:main'
        ]
    },
    classifiers=[
        'Programming Language :: Python :: 3.12',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)